@@ -25,10 +25,9 @@
 #include "Index.h"
 #include "Leaf.h"
 
+#include <cstring>
+
 using namespace SpatialIndex::MVRTree;
-typedef SpatialIndex::IShape IShape;
-typedef SpatialIndex::id_type id_type;
-typedef SpatialIndex::TimeRegion TimeRegion;
 
 //
 // Tools::IObject interface
@@ -203,6 +202,22 @@ void Node::getChildShape(size_t index, IShape** out) const
 	*out = new TimeRegion(*(m_ptrMBR[index]));
 }
 
+
+void Node::getChildData(size_t index, size_t& length, byte** data) const
+{
+        if (index < 0 || index >= m_children) throw Tools::IndexOutOfBoundsException(index);
+        if (m_pData[index] == NULL) 
+        {
+                length = 0;
+                data = NULL;
+        } 
+        else
+        {
+                length = m_pDataLength[index];
+                *data = m_pData[index];
+        }
+}
+
 size_t Node::getLevel() const
 {
 	return m_level;
@@ -236,7 +251,7 @@ Node::Node() :
 {
 }
 
-Node::Node(MVRTree* pTree, id_type id, size_t level, size_t capacity) :
+	Node::Node(SpatialIndex::MVRTree::MVRTree* pTree, id_type id, size_t level, size_t capacity) :
 	m_pTree(pTree),
 	m_level(level),
 	m_identifier(id),
