@@ -19,8 +19,7 @@
 //  Email:
 //    mhadji@gmail.com
 
-#ifndef __spatialindex_rtree_leaf_h
-#define __spatialindex_rtree_leaf_h
+#pragma once
 
 namespace SpatialIndex
 {
@@ -31,7 +30,7 @@ namespace SpatialIndex
 		public:
 			virtual ~Leaf();
 
-		private:
+		protected:
 			Leaf(RTree* pTree, id_type id);
 
 			virtual NodePtr chooseSubtree(const Region& mbr, size_t level, std::stack<id_type>& pathBuffer);
@@ -46,5 +45,3 @@ namespace SpatialIndex
 		}; // Leaf
 	}
 }
-
-#endif /*__spatialindex_rtree_leaf_h*/
