@@ -19,8 +19,7 @@
 //  Email:
 //    mhadji@gmail.com
 
-#ifndef __spatialindex_tprtree_leaf_h
-#define __spatialindex_tprtree_leaf_h
+#pragma once
 
 namespace SpatialIndex
 {
@@ -46,5 +45,3 @@ namespace SpatialIndex
 		}; // Leaf
 	}
 }
-
-#endif /*__spatialindex_tprtree_leaf_h*/
